﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Controls;

namespace _2011_04_26_2011_04_27_Praktikum
{
    class FileType
    {
        protected string extension;
        protected long numberOfBytes;

        public FileType(string extension, long size)
        {
            this.extension = extension;
            this.numberOfBytes = size;
        }

        public string Extension
        {
            get { return extension; }
        }

        public long NumberOfBytes
        {
            get { return numberOfBytes; }
        }

        public virtual void AddSize(long size)
        {
            numberOfBytes += size;
        }

        public virtual System.Windows.UIElement GetGraphics(double width, double height)
        {
            System.Windows.Shapes.Rectangle r = new System.Windows.Shapes.Rectangle();
            r.Width = width;
            r.Height = height;
            r.Fill = System.Windows.Media.Brushes.LightBlue;
            return r;
        }
    }

    class Pdf : FileType
    {
        public Pdf(long size)
            : base(".pdf", size)
        {
        }

        public override System.Windows.UIElement GetGraphics(double width, double height)
        {
            System.Windows.Shapes.Rectangle r = new System.Windows.Shapes.Rectangle();
            r.Width = width;
            r.Height = height;
            r.Fill = System.Windows.Media.Brushes.Red;
            return r;
        }
    }

    class Png : FileType
    {
        public Png(long size)
            : base(".png", size)
        {
            fileLengths.Add(size);
        }

        List<long> fileLengths = new List<long>();

        public override void AddSize(long size)
        {
            numberOfBytes += size;
            fileLengths.Add(size);
        }

        public override System.Windows.UIElement GetGraphics(double width, double height)
        {
            System.Windows.Shapes.Polyline p = new System.Windows.Shapes.Polyline();
            p.Stroke = System.Windows.Media.Brushes.Orange;
            p.StrokeThickness = 2.0;
            double y = 0.0;
            double x = 0.0;
            double sum = fileLengths.Sum();
            for (int i = 0; i < fileLengths.Count; i++)
            {
                p.Points.Add(new System.Windows.Point(x, y));
                y += height*fileLengths[i]/sum;
                p.Points.Add(new System.Windows.Point(x, y));
                x = width - x;
            }
            return p;
        }
    }

    class DirectoryStatistics
    {
        List<FileType> fileTypes = new List<FileType>();

        public DirectoryStatistics(string path)
        {
            addFiles(new System.IO.DirectoryInfo(path), 10);
            fileTypes.Sort((x, y) => -x.NumberOfBytes.CompareTo(y.NumberOfBytes));
        }

        void addFiles(System.IO.DirectoryInfo dir, int levels)
        {
            foreach (var item in dir.GetFiles())
            {
                long size = item.Length;
                string ext = item.Extension;

                FileType ft = fileTypes.Find(x => x.Extension == ext);

                if (ft != null)
                {
                    ft.AddSize(size);
                }
                else
                {
                    if (ext == ".png")
                    {
                        fileTypes.Add(new Png(size));
                    }
                    else if (ext == ".pdf")
                    {
                        fileTypes.Add(new Pdf(size));
                    }
                    else
                    {
                        fileTypes.Add(new FileType(ext, size));
                    }
                }                 
            }
            if (levels > 0)
            {
                foreach (var item in dir.GetDirectories())
                {
                    addFiles(item, levels - 1);
                }
            }
        }

        const double distanceFromBottom = 20.0;
        const double distanceToNeighbor = 2.0;

        public void DrawIn(Canvas canvas)
        {
            if (fileTypes.Count == 0)
            {
                return;
            }

            long maxNumberOfBytes = fileTypes.Max(x => x.NumberOfBytes);

            if (maxNumberOfBytes == 0)
            {
                return;
            }

            canvas.Children.Clear();
            for (int i = 0; i < fileTypes.Count; i++)
            {
                double width = 0.8 * canvas.Width / fileTypes.Count - distanceToNeighbor;
                
                // Max, damit mindestens ein Pixel hoch
                double height = Math.Max(1.0, (canvas.Height - distanceFromBottom) * fileTypes[i].NumberOfBytes / maxNumberOfBytes);            
                
                System.Windows.UIElement ui = fileTypes[i].GetGraphics(width, height);
                double x = i * canvas.Width / fileTypes.Count;
                Canvas.SetLeft(ui, x);
                Canvas.SetBottom(ui, distanceFromBottom);
                canvas.Children.Add(ui);

                Label label = new Label();
                label.Content = fileTypes[i].Extension;
                Canvas.SetLeft(label, x-5.0);
                Canvas.SetTop(label, canvas.Height - distanceFromBottom);
                canvas.Children.Add(label);
            }
        }
    }
}
